use crate::prelude::*;

use std::os::raw::c_void;
use std::ptr;

#[repr(C)]
pub struct VkRayTracingShaderGroupCreateInfoKHR {
    pub sType: VkStructureType,
    pub pNext: *const c_void,
    pub r#type: VkRayTracingShaderGroupTypeKHR,
    pub generalShader: u32,
    pub closestHitShader: u32,
    pub anyHitShader: u32,
    pub intersectionShader: u32,
    pub pShaderGroupCaptureReplayHandle: *const c_void,
}

impl VkRayTracingShaderGroupCreateInfoKHR {
    pub fn new(
        r#type: VkRayTracingShaderGroupTypeKHR,
        general_shader: u32,
        closest_hit_shader: u32,
        any_hit_shader: u32,
        intersection_shader: u32,
    ) -> Self {
        VkRayTracingShaderGroupCreateInfoKHR {
            sType: VK_STRUCTURE_TYPE_RAY_TRACING_SHADER_GROUP_CREATE_INFO_KHR,
            pNext: ptr::null(),
            r#type,
            generalShader: general_shader,
            closestHitShader: closest_hit_shader,
            anyHitShader: any_hit_shader,
            intersectionShader: intersection_shader,
            pShaderGroupCaptureReplayHandle: ptr::null(),
        }
    }

    pub fn set_replay_handle<T>(&mut self, replay_handle: &T) {
        self.pShaderGroupCaptureReplayHandle = replay_handle as *const T as *const _;
    }
}
